/*
 *  file      : src/ntb_package.c
 *  project   : b2im
 *  copyright : (C) 2015 by BULIN Claude
 * 
 *  This file is part of b2im project
 * 
 *  b2im is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; version 3.
 * 
 *  b2im is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif
#include <gtk/gtk.h>
#include <glib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_b2im/lib.h"
#endif

#include "global.h"
#include "configuser.h"





typedef struct {
	GtkWidget	*button;
	gchar		*txt;
	gboolean	bool_activate;
	gboolean	bool_sensitive;
} STRUCT_CHECK;

typedef struct {
	GtkWidget	*NtbPackage_button;
	gchar		*path_folder_manjaro_profiles_desktop_actif;
	GtkWidget	*adr_textview;
	gchar		*save_buf_desktop;
	size_t 		size_save_buf_desktop;
	GList		*list_check;
} VAR_PACKAGE;

VAR_PACKAGE VarPackage;




void NtbPackage_remove_list( void )
{
	GList	*list = NULL;
	STRUCT_CHECK	*StructCheck = NULL;

	if( NULL != ( list = g_list_first( VarPackage.list_check ))) {
		while (list) {
			if( NULL != (StructCheck = (STRUCT_CHECK *)list->data) ) {
				if( NULL != StructCheck->button )
					gtk_widget_destroy( StructCheck->button );
				if( NULL != StructCheck->txt )
					g_free( StructCheck->txt );
				g_free( StructCheck );
				StructCheck = list->data = NULL;
			}
			list = g_list_next( list );
		}
		g_list_free( VarPackage.list_check );
		VarPackage.list_check = NULL;
	}
}

gint NtbPackage_compar( STRUCT_CHECK * a, STRUCT_CHECK * b )
{
	return strcmp( (char*)a->txt, (char*)b->txt );
}

void NtbPackage_toggle_event( GtkToggleButton *togglebutton, gpointer user_data )
{
	STRUCT_CHECK *a = user_data;
	a->bool_activate = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(a->button) );
}

void NtbPackage_set_check( void )
{
	gchar		**Larrbuf = NULL;
	gchar		StrCapitalyse[ 30 ];
	gchar		*Ptr = NULL;
	gchar		*PathNameFile = NULL;
	gchar		*Buffer = NULL;
	FILE		*fp = NULL;
	size_t 		SizeFile;
	gint		Cpt;
	STRUCT_CHECK	*StructCheck = NULL;
	GList			*list = NULL;
	gchar		*info_label = NULL;
	
	if( NULL == VarPackage.NtbPackage_button ) return;
	
	// Split name5-desktop-truc
	Larrbuf = g_strsplit( NtbDesktop_get_desktop_actif(), "-", 0 );
	strcpy( StrCapitalyse, Larrbuf[ 0 ] );
	g_strfreev( Larrbuf );
	// Remove number in first name: 'name5' -> 'name'
	for( Ptr = StrCapitalyse; *Ptr; Ptr ++ )
		if( *Ptr >= '0' && *Ptr <= '9' ) {
			*Ptr = '\0';
			break;
		}
	// Capitalyse first letter
	StrCapitalyse[ 0 ] = g_ascii_toupper( StrCapitalyse[ 0 ] );
	
	PathNameFile = g_strdup_printf("%s/%s/Packages-%s",
						Config.path_folder_manjaro_profiles,
						NtbDesktop_get_desktop_actif(),
						StrCapitalyse
						);
	
	info_label = g_strdup_printf( _("<b>Package Selection from: [ Packages-%s ]</b>"), StrCapitalyse );
	gtk_label_set_label( GTK_LABEL(GLADE_GET_OBJECT("label35")), info_label );
	g_free( info_label );
	info_label = NULL;
	
	if( NULL == VarPackage.path_folder_manjaro_profiles_desktop_actif ) {
		VarPackage.path_folder_manjaro_profiles_desktop_actif = g_strdup( PathNameFile );
	} else {
		if( strcmp( PathNameFile, VarPackage.path_folder_manjaro_profiles_desktop_actif ) != 0 ) {
			g_free( VarPackage.path_folder_manjaro_profiles_desktop_actif );
			VarPackage.path_folder_manjaro_profiles_desktop_actif = NULL;
			VarPackage.path_folder_manjaro_profiles_desktop_actif = g_strdup( PathNameFile );
		}
		else {
			printf("%s*-*-> The name of the desktop has not changed%s\n", BOLD_YELLOW, RESET );
			return;
		}
	}
	
	NtbPackage_remove_list();
	
	if( FALSE == g_file_test( PathNameFile, G_FILE_TEST_EXISTS )) {
		// Si le nom ne correspond pas
		// DEMANDE A L UTILISATEUR DE SELECTIONNER LE FICHIER A OUVRIR
		//
		// If the name does not match
		// APPLICATION TO SELECT USER FILE TO OPEN
		//
		// 	filechooserdialog = gtk_file_chooser_dialog_new( ...
	}
		
	/**
	LECTURE DU FICHIER EN MEMOIRE
	*/
	if(( SizeFile = libutils_get_size_file( PathNameFile )) > 0 ) {
		if( NULL != ( fp = fopen( PathNameFile, "r" ))) {
			Buffer = (gchar *)g_malloc0( sizeof(gchar) * ( SizeFile +10 ));
			SizeFile = fread( Buffer, SizeFile, 1, fp );
			fclose( fp );
			Larrbuf = g_strsplit( Buffer, "\n", 0 );
			g_free( Buffer );
			Buffer = NULL;
		}
	}
	g_free( PathNameFile );
	PathNameFile = NULL;
	
	// Make List STRUCT_CHECK
	for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
		if( *Larrbuf[ Cpt ] == '#' ||
			// *Larrbuf[ Cpt ] == '>' ||
			*Larrbuf[ Cpt ] == '\n' ||
			*Larrbuf[ Cpt ] == '\0') continue;
		if( TRUE == libutils_str_is_empty( Larrbuf[ Cpt ] )) continue;
		StructCheck = (STRUCT_CHECK *)g_malloc0( sizeof(STRUCT_CHECK) );
		StructCheck->button         = NULL;
		StructCheck->txt            = g_strdup( Larrbuf[ Cpt ] );
		if( NULL != ( Ptr = strchr( StructCheck->txt, '#' ))) {
			Ptr --;
			while( *Ptr != ' ' && *Ptr != '\t' ) Ptr --;
			*Ptr = '\0';
		}
		StructCheck->bool_activate  = TRUE;
		StructCheck->bool_sensitive = ( *Larrbuf[ Cpt ] == '>' ) ? FALSE : TRUE;
		VarPackage.list_check = g_list_append( VarPackage.list_check, StructCheck );
	}
	g_strfreev( Larrbuf );
	
	// Sort List STRUCT_CHECK
	VarPackage.list_check = g_list_sort( VarPackage.list_check, (GCompareFunc)NtbPackage_compar );

	// Puts List STRUCT_CHECK
	if( NULL != ( list = g_list_first( VarPackage.list_check ))) {
		while( list ) {
			if( NULL != (StructCheck = (STRUCT_CHECK *)list->data) ) {
				StructCheck->button = gtk_check_button_new_with_label( StructCheck->txt );
				gtk_box_pack_start( GTK_BOX(VarPackage.NtbPackage_button), StructCheck->button, FALSE, FALSE, 0 );
				gtk_widget_show( StructCheck->button );
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(StructCheck->button), StructCheck->bool_activate );
				gtk_widget_set_sensitive( GTK_WIDGET(GTK_TOGGLE_BUTTON(StructCheck->button)), StructCheck->bool_sensitive );
				
				g_signal_connect( (gpointer)StructCheck->button, "toggled",  G_CALLBACK(NtbPackage_toggle_event), StructCheck );
			}
			list = g_list_next( list );
		}
	}
}

void on_box_packages_selection_realize( GtkWidget *widget, gpointer user_data )
{
	VarPackage.NtbPackage_button = widget;
}

gboolean NtbPackage_test_package( gchar *p_name )
{
	// gchar	*LC = g_strdup_printf( "pacman -Ss %s  &>/dev/null", p_name );
	gchar	*LineCommand = g_strdup_printf( "pacman -Ss %s", p_name );
	FILE	*fp;
	gchar buf[256];
	gboolean	bool_ret = FALSE;
	
	if( (fp = popen( LineCommand, "r" )) == NULL ) { 
		perror( "popen error" );
		exit( 1 );
	}
	while( fgets( buf, sizeof(buf)-1, fp ) != NULL ) {
		bool_ret = TRUE;
		// printf("BUF = %s", buf );
	}
	pclose( fp );
	g_free( LineCommand );
	LineCommand = NULL;
	return( bool_ret );
}

void NtbPackage_save_list( void )
{
	FILE			*fp = NULL;
	STRUCT_CHECK	*StructCheck = NULL;
	GList			*list = NULL;
	gchar			**Larrbuf;
	gint			Cpt;
	
	printf("%s*-*-> Save file in mem: %s%s\n", BOLD_YELLOW, VarPackage.path_folder_manjaro_profiles_desktop_actif, RESET );

	// LECTURE DU FICHIER EN MEMOIRE
	if(( VarPackage.size_save_buf_desktop = libutils_get_size_file( VarPackage.path_folder_manjaro_profiles_desktop_actif )) > 0 ) {
		if( NULL != ( fp = fopen( VarPackage.path_folder_manjaro_profiles_desktop_actif, "r" ))) {
			VarPackage.save_buf_desktop = (gchar *)g_malloc0( sizeof(gchar) * ( VarPackage.size_save_buf_desktop +10 ));
			fread( VarPackage.save_buf_desktop, VarPackage.size_save_buf_desktop, 1, fp );
			fclose( fp );
		}
	}

	printf("%s*-*-> Save data: %s%s\n", BOLD_YELLOW, VarPackage.path_folder_manjaro_profiles_desktop_actif, RESET );
	fp = fopen( VarPackage.path_folder_manjaro_profiles_desktop_actif, "w" );
	if( NULL != ( list = g_list_first( VarPackage.list_check ))) {
		while( list ) {
			if( NULL != (StructCheck = (STRUCT_CHECK *)list->data) ) {
				if( TRUE == StructCheck->bool_sensitive && TRUE == StructCheck->bool_activate ) {
					if( TRUE == NtbPackage_test_package( StructCheck->txt )) {
						printf("%s      %s %s\n", BOLD_YELLOW, StructCheck->txt, RESET );
						fprintf( fp, "%s\n", StructCheck->txt );
					}
					else {
						printf("%s      %s %s\n", BOLD_RED, StructCheck->txt, RESET );
					}
				}
				else if( TRUE == StructCheck->bool_activate ) {
					printf("%s      %s %s\n", BOLD_YELLOW, StructCheck->txt, RESET );
					fprintf( fp, "%s\n", StructCheck->txt );
				}
			}
			list = g_list_next( list );
		}
	}

	// Get added data
	Larrbuf = libutils_get_textview( GTK_WIDGET(GLADE_GET_OBJECT("textview_add_packages")) );
	// test data
	for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
		if( *Larrbuf[ Cpt ] != '\0' ) {
			if( TRUE == NtbPackage_test_package( (gchar *)Larrbuf[ Cpt ] )) {
				printf("  -> %s      %s %s\n", BOLD_YELLOW, (gchar *)Larrbuf[ Cpt ], RESET );
				fprintf( fp, "%s\n", (gchar *)Larrbuf[ Cpt ] );
			}
			else {
				printf("  -> %s      %s %s\n", BOLD_RED, (gchar *)Larrbuf[ Cpt ], RESET );
			}
		}
	}
	g_strfreev( Larrbuf );
	Larrbuf = NULL;

	fclose( fp );
}
void NtbPackage_restore_list( void )
{
	FILE			*fp = NULL;
	
	printf("%s*-*-> Restore file from mem: %s%s\n", BOLD_YELLOW, VarPackage.path_folder_manjaro_profiles_desktop_actif, RESET );
	if( NULL != ( fp = fopen( VarPackage.path_folder_manjaro_profiles_desktop_actif, "w" ))) {
		fwrite( VarPackage.save_buf_desktop, VarPackage.size_save_buf_desktop, 1, fp );
		fclose( fp );
	}
	g_free( VarPackage.save_buf_desktop );
	VarPackage.save_buf_desktop = NULL;
}

void NtbPackage_changed_textview( GtkTextBuffer *textbuffer, gpointer user_data )
{
	/*
	if( FALSE == VarPackage.BoolAccessChanged ) {
		// printf("changed_textview()\n" );
	}
	*/
}

void on_textview_add_packages_realize( GtkWidget *widget, gpointer user_data )
{
	VarPackage.adr_textview = widget;
	g_signal_connect( (gpointer)gtk_text_view_get_buffer(GTK_TEXT_VIEW (widget)), "changed", G_CALLBACK(NtbPackage_changed_textview), NULL);
}

// Vérifiez la présence des paquets dans les dépôts
// Check the packages in the repositories
void NtbPackage_thread_verify_packages( void )
{
	STRUCT_CHECK	*StructCheck = NULL;
	GList			*list = NULL;
	gchar			**Larrbuf;
	gint			Cpt;
	
	printf("%s*-*-> on_button_check_packages_clicked()%s\n", BOLD_YELLOW, RESET );
	// Check datas file
	if( TRUE == gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(GLADE_GET_OBJECT("checkbutton_package")) )) {
		if( NULL != ( list = g_list_first( VarPackage.list_check ))) {
			while( list ) {
				if( NULL != (StructCheck = (STRUCT_CHECK *)list->data) ) {
					if( TRUE == StructCheck->bool_sensitive && TRUE == StructCheck->bool_activate ) {
						if( TRUE == NtbPackage_test_package( StructCheck->txt )) {
							printf("%s      %s %s\n", BOLD_YELLOW, StructCheck->txt, RESET );
						}
						else {
							printf("%s      %s %s\n", BOLD_RED, StructCheck->txt, RESET );
						}
					}
				}
				list = g_list_next( list );
			}
		}
	}
	if( TRUE == gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(GLADE_GET_OBJECT("checkbutton_add_package")) )) {
		// Get added data
		Larrbuf = libutils_get_textview( GTK_WIDGET(GLADE_GET_OBJECT("textview_add_packages")) );
		// test data
		for( Cpt = 0; Larrbuf[ Cpt ]; Cpt++ ) {
			if( *Larrbuf[ Cpt ] != '\0' ) {
				if( TRUE == NtbPackage_test_package( (gchar *)Larrbuf[ Cpt ] )) {
					printf("  -> %s      %s %s\n", BOLD_YELLOW, (gchar *)Larrbuf[ Cpt ], RESET );
				}
				else {
					printf("  -> %s      %s %s\n", BOLD_RED, (gchar *)Larrbuf[ Cpt ], RESET );
				}
			}
		}
		g_strfreev( Larrbuf );
		Larrbuf = NULL;
	}
	pulse_stop();
}

void NtbPackage_timeout_begin( void )
{
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("box16")), FALSE );
}
void NtbPackage_timeout_end( void )
{
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("box16")), TRUE );
}

void on_button_check_packages_clicked( GtkButton *widget, gpointer user_data )
{
	pulse_run(
		"Verification of the presence of the packages in the database, please be patient ...",
		NtbPackage_timeout_begin,
		NtbPackage_timeout_end,
		NtbPackage_thread_verify_packages
		);

}



